class Vector
  class DimensionMismatchError < StandardError; end

  attr_reader :dimension

  def initialize(data)
    @dimension = data.size
    @data = data
  end

  def +(other)
    operator(:+, other) do |other_data|

      new_data = []
      other_data.each_with_index do |e, i|
        new_data[i] = @data[i] + e
      end

      Vector.new new_data
    end
  end

  def *(other)
    operator(:*, other) do |other_data|

      new_data = []
      other_data.each_with_index do |e, i|
        new_data << e + @data[i]
      end

      new_data.inject(:+)
    end
  end

  # def +(other)
  #   if other.kind_of? Vector
  #     raise DimensionMismatchError, "Dimension mismatch" unless other.dimension == self.dimension
  #     other_data = other.to_a

  #     new_data = []
  #     other_data.each_with_index do |e, i|
  #       new_data[i] = @data[i] + e
  #     end

  #     Vector.new new_data
  #   elsif other.kind_of? Numeric
  #     Vector.new @data.map { |e| other + e }
  #   end
  # end

  # def *(other)
  #   if other.kind_of? Vector
  #     raise DimensionMismatchError, "Dimension mismatch" unless other.dimension == self.dimension
  #     other_data = other.to_a

  #     new_data = []
  #     other_data.each_with_index do |e, i|
  #       new_data << e + @data[i]
  #     end

  #     new_data.inject(:+)
  #   elsif other.kind_of? Numeric
  #     Vector.new @data.map { |e| other * e }
  #   end
  # end

  def operator(op, other)
    if other.kind_of? Vector
      raise DimensionMismatchError, "Dimension mismatch" unless other.dimension == self.dimension
      other_data = other.to_a

      yield other_data

    elsif other.kind_of? Numeric
      Vector.new @data.map { |e| other.send(op, e) }
    else
      raise TypeError, "Don't know how to do #{op} with #{other.class}."
    end
  end

  def each
    @data.each do |e|
      yield e
    end
  end

  def map
    Vector.new @data.map { |e| yield e }
  end

  def -(other)
    self + (other*-1)
  end

  def to_a
    @data.dup
  end

  def to_s
    "(#{@data.join(", ")})"
  end
end

a = Vector.new [1, 2, 3]
b = Vector.new [4, 5, 6]

puts "a: #{a}"
puts "b: #{b}"

puts "a + b: #{a + b}"
puts "a - b: #{a - b}"
puts "a * b: #{a * b}"

puts "a + 3: #{a + 3}"
puts "b + 3: #{b + 3}"

puts "a.map { |e| e * e}: #{a.map { |e| e * e}}"
